<?php

namespace FireDrum;

if ( !defined( 'ABSPATH' ) ) {
	die();
}

class Email_Marketing_API {
	private $url;

	private $key;

	private $transient_expiration;

	private $plugin;

	public function __construct($plugin = null) {
		$this->plugin = ( $plugin === null ? Email_Marketing_Plugin::instantiate() : $plugin );
		$this->url = get_option( Email_Marketing_Plugin::get_branding() . '_api_url', 'api.' . Email_Marketing_Plugin::get_branding_domain() . '/api/v2.jsp' );
		$this->key = get_option( Email_Marketing_Plugin::get_branding() . '_api_key' );
		$this->transient_expiration = 86400;
	}

	public function filter_user_agent($user_agent) {
		return Email_Marketing_Plugin::get_menu_branding() . '_Email_Marketing_Plugin/' . Email_Marketing_Plugin::get_version() . "; {$user_agent}";
	}

	public function set_url($url) {
		$this->url = $url;
	}

	public function get_url() {
		return $this->url;
	}

	public function set_key($key) {
		$this->key = $key;
	}

	public function get_key() {
		return $this->key;
	}

	public function get_public_app_properties_retrieve_url($url = '') {
		if ( $url === '' ) {
			$url = $this->url;
		}

		return "https://{$url}?action=PublicAppProperties_Retrieve";
	}

	public function get_category_list_url($url = '') {
		if ( $url === '' ) {
			$url = $this->url;
		}

		return "https://{$url}?action=Category_List";
	}

	public function get_add_category_url($url = '') {
		if ( $url === '' ) {
			$url = $this->url;
		}

		return "https://{$url}?action=Category_Add";
	}

	public function get_custom_field_list_url($url = '') {
		if ( $url === '' ) {
			$url = $this->url;
		}

		return "https://{$url}?action=CustomField_List";
	}

	public function get_add_contact_url($url = '') {
		if ( $url === '' ) {
			$url = $this->url;
		}

		return "https://{$url}?action=Member_Add";
	}

	public function get_campaign_list_url($url = '') {
		if ( $url === '' ) {
			$url = $this->url;
		}

		return "https://{$url}?action=Campaign_List";
	}

	public function get_send_campaign_url($url = '') {
		if ( $url === '' ) {
			$url = $this->url;
		}

		return "https://{$url}?action=Campaign_Send";
	}

	public function check_api_key($api_key = '') {
		$result = 8;

		$save_key = $this->key;
		if ( $api_key !== '' ) {
			$this->key = $api_key;
		}

		$data = $this->get_categories_from_api();
		if ( isset( $data['responseCode'] ) && isset( $data['responseCode']['id'] ) ) {
			$result = $data['responseCode']['id'];
		}

		if ( $api_key !== '' ) {
			$this->key = $save_key;
		}

		return $result !== 8;
	}

	public function add_category($category_name) {
		add_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );
		$response = wp_remote_post( esc_url_raw( $this->get_add_category_url() ), array (
			'timeout' => 60,
			'httpversion' => '1.1',
			'headers' => array (
				'Connection' => 'close'
			),
			'body' => 'authAPIKey=' . rawurlencode( $this->key ) . '&publicDisplay=false&name=' . rawurlencode( $category_name )
		) );
		remove_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );

		return json_decode( wp_remote_retrieve_body( $response ), TRUE );
	}

	public function get_categories_from_api() {
		if ( ( $data = get_transient( Email_Marketing_Plugin::get_branding() . '_categories_response' ) ) === false ) {
			add_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );
			$response = wp_remote_post( esc_url_raw( $this->get_category_list_url() ), array (
				'timeout' => 60,
				'httpversion' => '1.1',
				'headers' => array (
					'Connection' => 'close'
				),
				'body' => 'authAPIKey=' . $this->key
			) );
			remove_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );

			$data = json_decode( wp_remote_retrieve_body( $response ), TRUE );
			if ( isset( $data['responseCode'] ) && isset( $data['responseCode']['id'] ) && $data['responseCode']['id'] === 0 ) {
				set_transient( Email_Marketing_Plugin::get_branding() . '_categories_response', $data, $this->transient_expiration );
			}
		}
		return $data;
	}

	public function get_campaigns_from_api() {
		if ( ( $data = get_transient( Email_Marketing_Plugin::get_branding() . '_campaigns_response' ) ) === false ) {
			add_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );
			$response = wp_remote_post( esc_url_raw( $this->get_campaign_list_url() ), array (
				'timeout' => 60,
				'httpversion' => '1.1',
				'headers' => array (
					'Connection' => 'close'
				),
				'body' => 'authAPIKey=' . $this->key
			) );
			remove_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );

			$data = json_decode( wp_remote_retrieve_body( $response ), TRUE );
			if ( isset( $data['responseCode'] ) && isset( $data['responseCode']['id'] ) && $data['responseCode']['id'] === 0 ) {
				set_transient( Email_Marketing_Plugin::get_branding() . '_campaigns_response', $data, $this->transient_expiration );
			}
		}
		return $data;
	}

	public function get_custom_fields_from_api() {
		if ( ( $data = get_transient( Email_Marketing_Plugin::get_branding() . '_custom_fields_response' ) ) === false ) {
			add_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );
			$response = wp_remote_post( esc_url_raw( $this->get_custom_field_list_url() ), array (
				'timeout' => 60,
				'httpversion' => '1.1',
				'headers' => array (
					'Connection' => 'close'
				),
				'body' => 'authAPIKey=' . $this->key
			) );
			remove_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );

			$data = json_decode( wp_remote_retrieve_body( $response ), TRUE );
			if ( isset( $data['responseCode'] ) && isset( $data['responseCode']['id'] ) && $data['responseCode']['id'] === 0 ) {
				set_transient( Email_Marketing_Plugin::get_branding() . '_custom_fields_response', $data, $this->transient_expiration );
			}
		}
		return $data;
	}

	public function get_public_app_properties_from_api() {
		if ( ( $data = get_transient( Email_Marketing_Plugin::get_branding() . '_app_properties_response' ) ) === false ) {
			add_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );
			$response = wp_remote_post( esc_url_raw( $this->get_public_app_properties_retrieve_url() ), array (
				'timeout' => 60,
				'httpversion' => '1.1',
				'headers' => array (
					'Connection' => 'close'
				),
				'body' => 'authAPIKey=' . $this->key
			) );
			remove_filter( 'http_headers_useragent', array (
				$this,
				'filter_user_agent'
			) );

			$data = json_decode( wp_remote_retrieve_body( $response ), TRUE );
			if ( isset( $data['responseCode'] ) && isset( $data['responseCode']['id'] ) && $data['responseCode']['id'] === 0 ) {
				set_transient( Email_Marketing_Plugin::get_branding() . '_app_properties_response', $data, $this->transient_expiration );
			}
		}
		return $data;
	}

	public function add_contact($email, $reCaptchaEnabled = true, $form_options) {
		$params = 'authAPIKey=' . rawurlencode( $this->key ) . '&updateIfExists=true&reactivationEnabled=true&subscribeMethod=wordPress&email=' . rawurlencode( $email );

		if ( isset( $_POST['fname'] ) ) {
			$params .= '&firstName=' . rawurlencode( $_POST['fname'] );
		}
		if ( isset( $_POST['lname'] ) ) {
			$params .= '&lastName=' . rawurlencode( $_POST['lname'] );
		}
		if ( isset( $_POST['company'] ) ) {
			$params .= '&company=' . rawurlencode( $_POST['company'] );
		}
		if ( isset( $_POST['address'] ) ) {
			$params .= '&address=' . rawurlencode( $_POST['address'] );
		}
		if ( isset( $_POST['address2'] ) ) {
			$params .= '&address2=' . rawurlencode( $_POST['address2'] );
		}
		if ( isset( $_POST['city'] ) ) {
			$params .= '&city=' . rawurlencode( $_POST['city'] );
		}
		if ( isset( $_POST['state'] ) ) {
			$params .= '&state=' . rawurlencode( $_POST['state'] );
		}
		if ( isset( $_POST['zip'] ) ) {
			$params .= '&zip=' . rawurlencode( $_POST['zip'] );
		}
		if ( isset( $_POST['phone'] ) ) {
			$params .= '&phone=' . rawurlencode( $_POST['phone'] );
		}
		if ( isset( $_POST['mphone'] ) ) {
			$params .= '&mobilePhone=' . rawurlencode( $_POST['mphone'] );
		}
		if ( isset( $_POST['fax'] ) ) {
			$params .= '&fax=' . rawurlencode( $_POST['fax'] );
		}
		if ( isset( $_POST['gender'] ) ) {
			$params .= '&gender=' . rawurlencode( $_POST['gender'] );
		}
		if ( $_POST['bdaymonth'] != "" || $_POST['bdayday'] != "" ) {
			$params .= '&birthDate=' . rawurlencode( $_POST['bdaymonth'] . '-' . $_POST['bdayday'] );
		}
		if ( $_POST['annvmonth'] != "" || $_POST['annvday'] != "" ) {
			$params .= '&anniversaryDate=' . rawurlencode( $_POST['annvmonth'] . '-' . $_POST['annvday'] );
		}
		if ( isset( $_POST['are_you_human_54'] ) ) {
			$params .= '&are_you_human_54=' . rawurlencode( $_POST['are_you_human_54'] );
		}
		
		$categories = $this->get_categories_from_api();
		if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) ) {
			foreach ( $categories['categories'] as $category ) {
				if ( in_array( $category['id'], $form_options['lists'] ) && in_array( $category['id'], $_POST['interests'] ) ) {
					$params .= '&categoryId[]=' . rawurlencode( $category['id'] );
				}
			}
		}

		$custom_fields = $this->get_custom_fields_from_api();
		if ( isset( $custom_fields['customFields'] ) && is_array( $custom_fields['customFields'] ) ) {
			foreach ( $custom_fields['customFields'] as $custom_field ) {
				if ( in_array( $custom_field['id'], $form_options['custom_fields'] ) ) {
					if ( isset( $_POST["field#_{$custom_field['id']}"] ) ) {
						$params .= '&customField[]=' . rawurlencode( json_encode( array (
							'id' => $custom_field['id'],
							'value' => $_POST["field#_{$custom_field['id']}"]
						) ) );
					} else if ( isset( $_POST['custmonth#_' . str_replace( " ", "_", $custom_field['name'] )] ) && isset( $_POST['custday#_' . str_replace( " ", "_", $custom_field['name'] )] ) ) {
						$params .= '&customField[]=' . rawurlencode( json_encode( array (
							'id' => $custom_field['id'],
							'value' => '0000-' . $_POST['custmonth#_' . str_replace( " ", "_", $custom_field['name'] )] . '-' . $_POST['custday#_' . str_replace( " ", "_", $custom_field['name'] )]
						) ) );
					}
				}
			}
		}

		if ( !$reCaptchaEnabled ) {
			$params .= '&skipCaptcha=true';
		} else if ( isset( $_POST['g-recaptcha-response'] ) ) {
			$params .= '&g-recaptcha-response=' . rawurlencode( $_POST['g-recaptcha-response'] );
		}
		
		
		if ( $form_options['double_opt_in'] === 'Y' ) {
			$params .= '&optInConfirmationEnabled=true';
		}
		
		add_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );
		$response = wp_remote_post( esc_url_raw( $this->get_add_contact_url() ), array (
			'timeout' => 60,
			'httpversion' => '1.1',
			'headers' => array (
				'Connection' => 'close'
			),
			'body' => $params
		) );
		remove_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );

		return json_decode( wp_remote_retrieve_body( $response ), TRUE );
	}

	public function add_contact_to_lists($email, $lists, $fields = array()) {
		$fields = wp_parse_args( $fields, array(
			'firstName' => '',
			'lastName' => '',
			'company' => '',
			'address' => '',
			'address2' => '',
			'city' => '',
			'state' => '',
			'zip' => '',
			'phone' => '',
			'mobilePhone' => '',
			'fax' => '',
			'gender' => '',
			'birthDate' => '',
			'anniversaryDate' => ''
		) );

		$params = 'authAPIKey=' . rawurlencode( $this->key ) . '&updateIfExists=true&reactivationEnabled=true&subscribeMethod=wordPress&skipCaptcha=true&email=' . rawurlencode( $email );

		foreach( $fields as $fieldName => $value ) {
			if ( is_array( $value ) ) {
				if ( $fieldName === 'customFields' ) {
					foreach ( $value as $custom_field ) {
						$params .= '&customField[]=' . rawurlencode( json_encode( $custom_field ) );
					}
				}
			} else if ( $value !== '' ) {
				$params .= '&' . $fieldName . '=' . rawurlencode( $value );
			}
		}

		$categories = $this->get_categories_from_api();
		if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) ) {
			foreach ( $categories['categories'] as $category ) {
				if ( in_array( $category['id'], $lists ) ) {
					$params .= '&categoryId[]=' . rawurlencode( $category['id'] );
				}
			}
		}

		add_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );
		$response = wp_remote_post( esc_url_raw( $this->get_add_contact_url() ), array (
			'timeout' => 60,
			'httpversion' => '1.1',
			'headers' => array (
				'Connection' => 'close'
			),
			'body' => $params
		) );
		remove_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );
		$json = wp_remote_retrieve_body( $response );
		return json_decode( $json, TRUE );
	}

	public function send_campaign_to_contact($campaign_id, $email) {
		add_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );
		$response = wp_remote_post( esc_url_raw( $this->get_send_campaign_url() ), array (
			'timeout' => 60,
			'httpversion' => '1.1',
			'headers' => array (
				'Connection' => 'close'
			),
			'body' => 'authAPIKey=' . rawurlencode( $this->key ) . '&campaignId=' . rawurlencode( $campaign_id ) . '&memberEmail=' . rawurlencode( $email )
		) );
		remove_filter( 'http_headers_useragent', array (
			$this,
			'filter_user_agent'
		) );
		$json = wp_remote_retrieve_body( $response );
		return json_decode( $json, TRUE );
	}
}
